#include "excalibur.h"  // for debug support, memory addresses and peripheral structures

#define NONE_PRESSED 0xF  // Value read from button PIO when no buttons pressed
#define DEBOUNCE 30  // Time in Milliseconds to wait for switch debounce

int main(void)
{
	int buttons;  // Use to hold button pressed value
	int led = 0x01;  // Use to write to led

	np_pio *buttonpio = na_button_pio;   // create variable buttonpio of type np_pio with addr na_button_pio
	np_pio *ledpio = na_led_pio;

	printf("Simple\n");   // print a message to show that program is running

	ledpio->np_piodata = led;  // write initial value to pio

	while (1)
	{
		buttons = buttonpio->np_piodata;   // read buttons via pio

		if (buttons != NONE_PRESSED)  // if button pressed
		{
			if (led >= 0x80)  // if pattern is 00000001 on board (leds in reverse order)
				led = 0x01;  // reset pattern
			else
				led = led << 1;  // shift right on board (led0 is far left)

			ledpio->np_piodata = led;  // write new value to pio

			/* Switch debounce routine
			/* Wait for small delay after intial press for debouncing
			/* Wait for release of key
			/* Wait for small delay after release for debouncing */

			nr_delay (DEBOUNCE);
			while (buttons != NONE_PRESSED)  // wait for button release
				buttons = buttonpio->np_piodata;   // update
			nr_delay (DEBOUNCE);
		}

	}


}  // end

